# -*- coding: utf-8 -*-
"""
Created on Wed Dec  2 14:27:28 2015

@author: Pierre
"""
##----- Importation des Modules -----##
import time
from tkinter import *
from tkinter.messagebox import *
from math import sqrt

##----- Définition des Fonctions -----##

def charge_problemes(niveau):
    """Renvoie le liste des problèmes du fichier niveau."""

    L = []    
    fichier_problemes = open(niveau, 'r')
    lignes = fichier_problemes.readlines()
    fichier_problemes.close()
    n = len(lignes)
    
    for i in range(n // 3):
        S = []      # On reconstitue la liste des sommets du problème indicé par i
        chaine_SOMMETS = lignes[3*i + 1].strip().strip('; ')
        for chaine_sommet in chaine_SOMMETS.split('; '):
            S.append((int(chaine_sommet.split(', ')[0]), int(chaine_sommet.split(', ')[1])))
            
        A = []      # On reconstitue la liste des aretes du problème indicé par i
        chaine_ARETES = lignes[3*i + 2].strip().strip('; ')
        for chaine_arete in chaine_ARETES.split('; '):
            A.append((int(chaine_arete.split(', ')[0]), int(chaine_arete.split(', ')[1])))
            
        L.append((S, A))
        
    return L

def liste_adj(A, nb_sommets):
    """Renvoie la liste d'adjacence correspondant à la liste A des aretes.
    Attention la liste d'adjacence renvoyée comporte, pour chaque sommet 
    lui-même comme voisin."""
    
    L = [[i] for i in range(nb_sommets)]    # On initialise : chaque sommet est 
                                            # son propre voisin. 
    for (n, p) in A:
        L[n].append(p)
        L[p].append(n)

    return L

def distance(P, Q):
    """P et Q sont deux tuples qui sont les coordonnées de deux points.
    Renvoie la distance entre P et Q."""
    
    return sqrt((Q[0] - P[0])**2 + (Q[1] - P[1])**2)

def sommet_proche(sommets, P):
    """sommets contient une liste de tuples qui sont les coordonnées des
    sommets, P est un tuple de coordonnées et rayon un float.
    Renvoie None si P est à distance > rayon de tous les sommets.
    Renvoie le numéro du sommet qui est à une distance <= rayon sinon"""

    for i in range(len(sommets)):
        M = sommets[i]
        if distance(P, M) <= rayon_sommet:
            return i
    
    return None

def dessine_sommets(sommets):
    """figure est un widget de type canvas, sommets est une liste de tuples
    qui sont les coordonnées des sommets et rayon est un float.
    Crée des items de type <<create_oval>> aux positions données par sommets
    et retourne la liste des identifiants correpondants"""

    identifiants_sommets = []
    for (x, y) in sommets:
        r = rayon_sommet
        identifiants_sommets.append(figure.create_oval(x - r, y - r, x + r, y + r,
        outline = 'black', fill = 'white', width = 4))
        
    return identifiants_sommets

def dessine_aretes(aretes, sommets):
    """Construit toutes les arêtes de la liste aretes dans le canvas figure"""
    
    for (n, p) in aretes:
        (x1, y1) = sommets[n]
        (x2, y2) = sommets[p]
        figure.create_line(x1, y1, x2, y2, width = 4, fill = 'black')
 
def change_sommet(k):
    """Change l'état du sommet k dans la figure et modifie l'élément
    correspondant dans liste_etat. C'est un effet de bord désiré"""

    if liste_etat[k] == 0:
        figure.itemconfig(identifiants_sommets[k], fill = 'black')
        liste_etat[k] = 1
    else:
        figure.itemconfig(identifiants_sommets[k], fill = 'white')
        liste_etat[k] = 0

    
def joue_sommet(k):
    """Effectue les opérations consécutives à un coup joué sur le sommet k"""
        
    global nb_coups, en_jeu, coups   
    
    nb_coups += 1       # met à jour le nombre de coups et l'affichage correspondant
    ch= "Coups joués : {:3} / {}"
    texte_nb_coups.set(ch.format(str(nb_coups), str(nb_mini_coups)))
    coups.append(k)     # met à jour la liste des coups
    cache_indication()  # cache une éventuelle indication donnée
    
    for i in l_adj[k]:   # agit sur le sommet et ses voisins
        change_sommet(i)
    if voir_sol:        # actualise la solution si elle est affichée
        actualise_couleur_solution()
    if sum(liste_etat) == len(sommets):  # teste si problème est réusi
        affichage_fellicitation()       # si c'est le cas on affiche les fellicitations
        en_jeu = False                  # baisse le drapeau indiquant que l'on est en jeu

def montre_indication():
    """ Fonction appelée lors d'un clic sur le bouton < Voir un coup > """
    
    global un_coup
    
    if not(en_jeu):     # Si on n'est pas en jeu le bouton est inactif
        return
        
    sol = meilleur_solution(liste_etat, l_adj)   # Calcule une meilleur solution
    un_coup = sol.index(1)                      # un_coup est le numéro d'un sommet à jouer
    figure.itemconfig(identifiants_sommets[un_coup], outline = 'red')
    
def cache_indication():
    """ Fonction qui sert à cacher l'indication si elle n'est plus d'actualité """
    
    global un_coup
    
    if un_coup == -1:       # Si il n'y a pas d'indication active on ne fait rien
        return
    figure.itemconfig(identifiants_sommets[un_coup], outline = 'black') # passe le bord en noir
    un_coup = -1        # baisse le drapeau d'indication
    
def montrer_solution():
    """ Fonction appelée lors d'un clic sur le bouton < Voir / Cacher la solution > """
    
    global voir_sol, solution        
    
    if not(en_jeu):     # Si on n'est pas en jeu le bouton est inactif
        return
    
    if voir_sol:
        voir_sol = False                # baisse le drapeau
        texte_bouton_sol.set(' Voir la solution ')      # Change le texte du bouton
        actualise_couleur_solution()
        
    else:
        voir_sol = True             # lève le drapeau
        solution = meilleur_solution(liste_etat, l_adj) # Calcule une meilleure solution
        actualise_couleur_solution()
        texte_bouton_sol.set(' Cacher la solution ')    # Change le texte du bouton
        

def actualise_couleur_solution():
    """ actualise la couleur des bords des sommets en fonction de l'état du drapeau
    voir_sol. """
    
    global solution
    
    solution = meilleur_solution(liste_etat, l_adj)     # actualise la solution
    n = len(sommets)
    
    for i in range(n):
        if solution[i] == 1 and voir_sol :
            figure.itemconfig(identifiants_sommets[i], outline = 'red')
        else:
            figure.itemconfig(identifiants_sommets[i], outline = 'black')

def retour():
    """ fonction appelée lors d'un clic sur le bouton retour """
    
    global nb_coups, coups
    
    if probleme_choisi == (-1, 0): # Si aucun problème le bouton est inactif
        return

    cache_indication()   # on cache une éventuelle indication 
    if nb_coups != 0:
        joue_sommet(coups[nb_coups - 1]) # rejout le coups indexé par nb_coups
        nb_coups = nb_coups - 2         # actualise le nb de coups (sic -2)
        coups.pop()     # pour que le  coups joué à l'envers ne s'ajoute pas à la liste
        ch= "Coups joués : {:3} / {}"   # actualise le texte des coups joués
        texte_nb_coups.set(ch.format(str(nb_coups), str(nb_mini_coups)))        
        
        
def avance():
    """ fonction appelée lors d'un clic sur le bouton avance """
    
    cache_indication()     # on cache une éventuelle indication 

    if probleme_choisi == (-1, 0): # Si aucun problème le bouton est inactif
        return
    
    if nb_coups < len(coups):
        joue_sommet(coups[nb_coups])
        coups.pop()      # pour que le  coups joué ne s'ajoute pas à la liste

def action_menu():
    """ Fonction appelée lors d'une action dans le menu """
    
    global  probleme_choisi, en_jeu, voir_sol

    S = 0
    for controles in controles_menu:
        S += sum([C.get() for C in controles])
    
    if S == 0:      # Si tout est décoché on ré-initialise 
        figure.delete("all")
        probleme_choisi = (-1, 0)
        en_jeu = False
        voir_sol = False
        texte_bouton_sol.set(' Voir la solution ')
        ch= "Coups joués : {:3}"
        texte_nb_coups.set(ch.format(str(0)))
        texte_fellicitation.set(" ")
        return
    else:           # sinon on décoche l'ancien choix
        controles_menu[probleme_choisi[0]][probleme_choisi[1]].set(0)
        
    # On met à jour la variable globale probleme_choisi 
    for i in range(len(controles_menu)):
        for j in range(len(controles_menu[i])):
            if controles_menu[i][j].get() == 1:
                probleme_choisi =(i, j)

    voir_sol = False
    texte_bouton_sol.set(' Voir la solution ')
    en_jeu = True
    initialise_probleme(probleme_choisi)

def affiche_regle():
    """ Fonction appelée lors d'un clic sur le bouton regle """

    showinfo("Régle du jeu",
             "En cliquant sur un sommet il change de couleur : blanc devient noir et réciproquement." +
             " Les sommets voisins sont modifiés en même temps.\n" +
             " Il faut faire passer tous les sommets de la couleur blanc à la couleur noir")

def affichage_fellicitation():
    """ Provoque l'affichage d'un message de fellicitation"""
    
    ch = "Bravo ! Vous avez résolu le problème en {} coups."
    texte_fellicitation.set(ch.format(nb_coups))
   
def centrer_sommets():
    """Centre les coordonnées de la liste des sommets. C'est une action en place"""  
    
    minX = min([s[0] for s in sommets]) # détermine les valeurs des bords de zone
    maxX = max([s[0] for s in sommets])
    minY = min([s[1] for s in sommets])
    maxY = max([s[1] for s in sommets])
        
    if maxX > largeur_figure:       # affinité sur les x si dépassement
        for i in range(len(sommets)):
            (x, y) = sommets[i]
            sommets[i] = (int(x*0.9*largeur_figure/maxX), y)
        minX = min([s[0] for s in sommets])  # recalcule les bords qui ont été modifiés
        maxX = max([s[0] for s in sommets])

    if maxY > hauteur_figure:       # affinité sur les y si dépassement
        for i in range(len(sommets)):
            (x, y) = sommets[i]
            sommets[i] = (x, int(y*0.9*hauteur_figure/maxY))
        minY = min([s[1] for s in sommets])  # recalcule les bords qui ont été modifiés
        maxY = max([s[1] for s in sommets])

    dX = (largeur_figure - maxX - minX) / 2  # Les translations pour centrer
    dY = (hauteur_figure - maxY - minY) / 2
    
    for i in range(len(sommets)):            # effectue les décalages 
        (x, y) = sommets[i]
        sommets[i] = (int(x + dX), int(y + dY))

        
def initialise_probleme(probleme_choisi):
    """Affiche, dans le canvas <figure>, le graphe correspondant au problème 
    désigné par la variable probleme_choisi et mets à jour de nombreuses variables 
    globales """

    global sommets, identifiants_sommets, l_adj, liste_etat, nb_coups 
    global nb_mini_coups, coups, un_coup

    figure.delete("all")
    nb_coups = 0
    coups = []
    un_coup = -1  # variable drapeau pour l'indication

    niveaux = ['Facile', 'Moyen', 'Difficile']
    L = charge_problemes(niveaux[probleme_choisi[0]] + 's.txt')
    (sommets, aretes) = L[probleme_choisi[1]]
    centrer_sommets()
    nb_sommets = len(sommets)
    dessine_aretes(aretes, sommets)    # les arêtes d'abord, elles seront dessous
    identifiants_sommets = dessine_sommets(sommets)  # dessine les sommets 
    
    l_adj = liste_adj(aretes, nb_sommets)
    liste_etat = [0 for _ in range(nb_sommets)]  
    texte_fellicitation.set(" ")        # remet à zéro le message de fellicitation
    
    sol = solutions_pivot(l_adj, [0 for _ in range(nb_sommets)])
    nb_mini_coups = min([sum(s) for s in sol])
    ch= "Coups joués : {:3} / {}"    
    texte_nb_coups.set(ch.format(str(nb_coups), str(nb_mini_coups)))

def recommence_probleme():
    """ fonction appelé lors d'un clic sur le bouton < Recommencer > """
    
    global en_jeu, voir_sol
    
    if probleme_choisi == (-1, 0): # Si aucun problème choisi le bouton est inactif
        return

    en_jeu = True
    voir_sol = False
    texte_bouton_sol.set(" Voir la solution ")
    initialise_probleme(probleme_choisi)
    
def initialise_menu():
    """ crée le cadre (frame) situé en haut de la fenêtre avec tous ses éléments 
    et retourne l'objet ainsi créé avec une liste de listes contenant les variables 
    de contrôle de tous les menus : par exemple controles_menu[1][3] est la variable 
    (type IntVar) correspondant au troisième choix du second menu (difficile 4)"""    
    
    # On charge d'abord les problèmes des trois niveaux
    problemes_niv = []
    niveaux = ['Facile', 'Moyen', 'Difficile']
    
    for nivo in niveaux:
        problemes_niv.append(charge_problemes(nivo+'s.txt'))
    
    menuframe = Frame(fenetre)    # Création d'un cadre (classe Frame) pour le menu
    # Création du bouton pour l'affichage de la règle
    menuregle = Button(menuframe, font = ('Helvetica', 24), text = 'Règle', 
                      command = affiche_regle)
    menuregle.pack(side = LEFT)
    
    # Création de la zone de texte pour anoncer les niveaux de difficultés
    titre_niveau = Label(menuframe, font = ('Helvetica', 24), text = 'Niveau de difficulté : ')
    titre_niveau.pack(side  = LEFT, padx = 10)
    
    # Création des boutons de menu pour les trois niveaux
    boutons_menu = []
    
    for nivo in niveaux:
        boutons_menu.append(Menubutton(menuframe, font = ('Helvetica', 20),
                                       text = nivo+'s', padx = 2, pady = 5))

    
    for bouton in boutons_menu:
      bouton.pack(side = LEFT, padx = 5, expand = YES)
        
    # Ajout des cases à cocher (Checkbutton) pour les menus
    controles_menu = []
    for i in range(len(problemes_niv)):
        S = []
        menu = Menu(boutons_menu[i])
        for j in range(len(problemes_niv[i])): 
            selection = IntVar()
            selection.set(0)
            menu.add_checkbutton(label = niveaux[i] + ' ' + str(j + 1), font = ('Helvetica', 20), 
                                 variable = selection, command = action_menu)
            S.append(selection)
        controles_menu.append(S)
        boutons_menu[i].config(menu = menu)
        
    return menuframe, controles_menu
    
##----- Fonctions permettant le calcul de l'ensemble des solutions d'un problème
    
def agir(L, E, k):
    """L est la liste d'adjacence qui représente le graphe d'un probleme, E
    la liste des états et k le numéro d'un sommet. Pour une action sur le
    sommet k, La fonction modifie E d'après L """
    
    for i in L[k]:
        E[i] = 1 - E[i]
    
def est_solution(L, A, etat_initial):
    """L est la liste d'adjacence qui représente le graphe d'un probleme,
    A est une liste de 0 et 1 qui indique pour chaque sommet si on agit
    sur celui-ci ou non, E est une liste qui donne l'état initial. L, A et E 
    sont de même longueur. La fonction renvoie un booléen qui indique si A 
    donne une solution au problème"""

    n = len(L)
    E = list(etat_initial) # pour éviter l'effet de bord de la fonction agir
    
    for i in range(n):
        if A[i] == 1:
            agir(L, E, i)
            
    return sum(E) == n
            
def binaire(x, n):
    """Renvoie la liste des n premiers bits de l'écriture binaire de x.
    Cette fonction sert dans la fonction 'solutions' à générer toutes les 
    actions possibles sur l'ensemble du graphe"""

    P = []
    for k in range(n):
        P.append(x%2)
        x = x//2
    return P

def solutions(L, etat_initial):
    """renvoie la liste de toutes les solutions pour un problème représenté par 
    la liste d'adj L et pour un état initial donné par etat_initial"""

    n = len(L)
    S = []
    
    for i in range(2**n):   # Boucle qui va générer toutes les actions possibles
                            # sur l'ensemble du graphe 
        A = binaire(i, n)   # A est l'action correspondant à l'entier i
        if est_solution(L, A, etat_initial):
            S.append(A)

    return S

def meilleur_solution(liste_etat, l_adj):
    """Renvoie la solution la plus proche de l'état donné par liste_etat : 
     c'est celle pour laquelle le nombre de coups à jouer est minimal."""
       
    liste_sol = solutions_pivot(l_adj, liste_etat)
    S_record = len(liste_etat) + 1  # le nombre de coups à jouer est au plus égal au nb de sommet 

    for sol in liste_sol:
        somme = sum(sol)
        if somme < S_record:
            meilleur_sol = sol
            S_record = somme
        
    return meilleur_sol

#  -------------  Ensemble des solutions à l'aide du pivot de Gauss  ------

def matrice_adj(l_adj):
    """ l_adj est une liste d'adjacence pour un graphe.
    Renvoie la matrice d'adjacence correspondant à la liste d'adjacence """

    n = len(l_adj)
    M = [[0 for j in range(n)] for i in range(n)]
    
    for i in range(n):
        for j in l_adj[i]:
            M[i][j] = 1
            
    return M

def affiche_matrice(M):
    """ fonction ayant servie pour la mise au point du code"""
    
    for i in range(len(M)):
        print(M[i])
    

def matrice_augmente(M, X):
    """M est une liste de listes représentant une matrice carrée et X une liste
    représentant un vecteur colonne. Renvoie la matrice obtenue en ajoutant
    la colonne X comme dernière colonne à la matrice M"""

    P = []
    n = len(X)
    
    for i in range(n):
        L = list(M[i])
        L.append(X[i])
        P.append(L)

    return P
        
def cherche_pivot(M, j, i):
    """ M est une liste de listes qui représente une matrice dont tous les
    coefficients sont dans {0, 1}. Cherche un pivot dans la colonne j de M à
    partir de la position i : Renvoie None si la colonne n'est constituée
    que de 0 à partir de i et sinon l'indice du premier 1 rencontré. """

    for k in range(i, len(M)):
        if M[k][j] == 1:
            return k
        
def permute_ligne(M, i1, i2):
    """permute les lignes i1 et i2 de M"""

    (M[i1], M[i2]) = (M[i2], M[i1])   # C'est une pythonerie honteuse mais tellement plus simple

def ajoute_ligne_Z2(M, i, k):
    """Ajoute dans Z/2Z la ligne k à la ligne i de la matrice M"""

    m = len(M[i])
    
    for j in range(m):
        M[i][j] = (M[i][j] + M[k][j])%2
   
def Gauss_Z2(M):
    """Applique la méthode du pivot de Gauss à la matrice M dans Z/2Z pour
    en faire une matrice échelonnée et renvoie la liste croissante des indices
    correspondant aux inconnues principales et aux inconnues secondaires"""

    n = len(M)  # nombre de lignes
    inconnues_p = []
    inconnues_s = []
    i = 0                   # indice de ligne pour la recherche du pivot
    for j in range(n):              # on balaie les colonnes
        k = cherche_pivot(M, j, i)  
        if k == None:               # si pas de pivot alors inconnue secondaire
            inconnues_s.append(j)
        else:                       # si pivot alors inconnue principale
            inconnues_p.append(j)
            permute_ligne(M, i, k)  # place le pivot
            for r in range(i + 1, n):  # traite les ligne sous le pivot
                if M[r][j] == 1:
                    ajoute_ligne_Z2(M, r, i)
            i += 1               # on incrémente l'indice pour chercher le prochain pivot   

    return (inconnues_p, inconnues_s)
    

def solutions_syst_Z2(M, Y):
    """M est une matrice carrée à coef dans Z/2Z.
    Renvoie l'ensemble des solutions dans Z/2Z du système MX=Y"""

    n = len(M)                      # nombre de lignes
    M_a = matrice_augmente(M, Y)
    # On échelonne M_a et récupère les indices des inconnues principales et secondaires
    (inconnues_p, inconnues_s) = Gauss_Z2(M_a)  
    r = len(inconnues_p)            # rang de la matrice
    # pas de solution si un élèment non nul est sur les dermières lignes dans le second membre
    for i in range(r, n):   
        if M_a[i][n] == 1:
            return []
    # P est la matrice de r lignes qui va permettre d'exprimer les inconnues principales
    # en fonction des secondaires. Les premières colonnes de P sont celles des inconnues
    # principales de M_a puis vient la dernière colonne de M_a et enfin les colonnes des
    # inconnues secondaires dans l'ordre.
    
    P = [[0 for _ in range(n + 1)] for _ in range(r)]
    # les colonnes des inconnues principales
    for j in range(r):
        k = inconnues_p[j] 
        for i in range(r):
            P[i][j] = M_a[i][k]
    # la colonne pour le second membre
    for i in range(r):
        P[i][r] = M_a[i][n]
    # les colonnes pour les inconnues secondaires
    for j in range(n - r):
        k = inconnues_s[j]
        for i in range(r):
                P[i][r + j + 1] = M_a[i][k]

    # On effectue un pivot sur les lignes au-dessus de la diagonale
    for j in range(1, r):
        for i in range(j):
            if P[i][j] == 1:
                ajoute_ligne_Z2(P, i, j)
    # il ne reste plus qu'à générer toutes les solutions           
    ens_sol = []
    for k in range(2**(n - r)):
        X_sec = binaire(k, n - r)       # génère des valeurs pour les inconnues secondaires
        Y = [P[i][r] for i in range(r)] # La solution particulière

        for i in range(r):              # Calcule les valeurs des inconnues principales
            for j in range(n - r):
                Y[i] = Y[i] + P[i][r + 1 + j]*X_sec[j]
            Y[i] = Y[i] % 2
        X = [0 for _ in range(n)]
        for i in range(r):              # place les valeurs des inconnues princpales
            X[inconnues_p[i]] = Y[i]
        for i in range(n-r):            # place les valeurs des inconnues secondaires
            X[inconnues_s[i]] = X_sec[i]
        ens_sol.append(X)               # ajoute la solution calculée à la liste
    return ens_sol
        
def solutions_pivot(L, etat_initial):
    """renvoie la liste de toutes les solutions pour un problème représenté par 
    la liste d'adj L et pour un état initial donné par etat_initial.
    Méthode matricielle par le pivot de Gauss"""

    M = matrice_adj(l_adj)
    n = len(L)
    # le second membre correspondant à l'état initial est obtenu par des compléments à 1
    Y = [1 - etat_initial[i] for i in range(n)]  
    
    return solutions_syst_Z2(M, Y)

##----- Fonctions réceptionnant les événements -----##
        
def clic_gauche(event):
    """ fonction appelé lors d'un clic gauche sur le canevas <figure> """
    global coups
        
    if not(en_jeu): # si on n'est pas en jeu, il ne se passe rien
        return
    k = sommet_proche(sommets, (event.x, event.y))  # si un sommet est joué on appelle la fonction
    if k != None:
        if nb_coups != len(coups):   # si un coups est joué alors qu'on navigue dans l'historique des coups 
            coups = coups[:nb_coups]  # les coups aprés sont retirés
        joue_sommet(k)
        
## --------  Variables globales  ------##

nb_coups = 0
en_jeu = False  # Drapeau qui indique si le joueur peux agir sur les sommets
voir_sol = False # Drapeau qui indique si on montre / cache la solution
probleme_choisi = (-1, 0)  # tuple qui donne le problème choisi par le joueur (0, 1)
                           # désigne par exemple le second problème du niveau facile
                           # et (-1,0) indique qu'aucun problème n'est choisi

# Création de la fenêtre principale (main window)

fenetre = Tk()
fenetre.title('Le jeu des bougies')

##----- paramètres de l'affichage -----##

largeur_figure = fenetre.winfo_screenwidth() // 2
hauteur_figure = fenetre.winfo_screenheight() // 2
rayon_sommet = int(25*fenetre.winfo_screenheight()/1440)

##----- Création d'une Frame pour le menu du jeu ----##

(menuframe, controles_menu) = initialise_menu()

##----- Création du canevas -----##

figure = Canvas(fenetre, width = largeur_figure, height = hauteur_figure, background='lightyellow')
# 'lightyellow' donne une couleur jaune pale au fond de la zone de jeu

##----- Boutons -----##

# Bouton "Retour" <
bou_retour = Button(fenetre, text = " Retour < ", command = retour)
bou_retour.config(font=('Helvetica', 20,  'bold'))

# Bouton "Avance" >
bou_avance = Button(fenetre, text = " Avance > ",  command = avance)
bou_avance.config(font=('Helvetica', 20, 'bold'))

# Bouton " Voir / Cacher la solution "
texte_bouton_sol = StringVar()
texte_bouton_sol.set(" Voir la solution ")  
bou_sol = Button(fenetre, textvariable = texte_bouton_sol, command = montrer_solution)
bou_sol.config(font=('Helvetica', 20, 'bold'))

# Bouton "voir un coup"
bou_un_coup = Button(fenetre, text = "Voir un coup" , command = montre_indication)
bou_un_coup.config(font=('Helvetica', 20, 'bold'))

# Bouton Recommencer

bou_recommencer = Button(fenetre, text = " Recommencer ", font=('Helvetica', 20, 'bold'),
                       command = recommence_probleme)

# Bouton " Quitter "

bou_quitter = Button(fenetre, text = " Quitter ", command = fenetre.destroy)
bou_quitter.config(font=('Helvetica', 20, 'bold'))

##----- Zones de texte ----- ##

texte_nb_coups = StringVar()
ch= "Coups joués : {:3} / {}"
nb_mini_coups = 0
texte_nb_coups.set(ch.format(str(nb_coups), str(nb_mini_coups)))
affichage_nb_coups = Label(fenetre, textvariable = texte_nb_coups, fg = 'blue', bg = 'yellow')
affichage_nb_coups.config(font=('Helvetica', 20, 'bold'))
texte_fellicitation = StringVar()
texte_fellicitation.set(" ")
message_fellicitation = Label(fenetre, textvariable = texte_fellicitation, fg = 'red')
message_fellicitation.config(font=('Helvetica', 20, 'bold'))

## ------ Placement des widgets ------ ##

menuframe.grid(row = 0, column = 1, columnspan = 4)
message_fellicitation.grid(row = 1, column = 1,  columnspan = 4, padx = 10)
figure.grid(row = 2, column = 1, columnspan = 4)
bou_retour.grid(row = 3, column = 2, padx = 10)
bou_avance.grid(row = 3, column = 3, padx = 10)
affichage_nb_coups.grid(row = 3, column = 4)
bou_un_coup.grid(row = 3, column = 1, padx = 10, pady = 5)
bou_sol.grid(row = 4, column = 1, padx = 10, pady = 5)
bou_recommencer.grid(row = 4, column = 2, columnspan = 2, padx = 10, pady = 5)
bou_quitter.grid(row = 4, column =4,  padx = 10, pady = 5)

##----- Evénements -----##

figure.bind("<Button-1>", clic_gauche)  # un clic gauche appelle la fonction clic_gauche

##----- Programme principal -----##

fenetre.mainloop()

try:
    fenetre.destroy()               # Fermeture de la fenêtre par le bouton
except TclError:
    pass


